//+------------------------------------------------------------------+
// This is an indicator; install into ...(MT4)/MQL4/Indicators folder

#property indicator_chart_window

#include <hanover --- function header (np).mqh>

extern string     UpLevels                   = "75,100,125";
extern string     DownLevels                 = ".";
extern int        StartCandle                = 10;
extern int        LineLength                 = 5;
extern int        ADRdays                    = 30;
extern color      UpLineColor                = Olive;
extern color      DownLineColor              = DarkGreen;
extern int        LineWidth                  = 1; 
extern int        LineStyle                  = STYLE_DASH; 
extern string     Visibility                 = "M1,M5,M15,M30,H1,H4,D1,W1,MN";
extern string     RefreshPeriod              = "M1";
extern bool       FixFromOpen                = false;
extern string     UniqueID                   = "0";
extern bool       UseGlobalVariables         = false;

string     ccy, sym, IndiName, diag_string, FontName, arr[10], gv_up, gv_dn, gv_op;
int        dig, tf, tmf, vis, wno, RefreshEveryXMins, bar, Window, Corner, HorizPos, VertPos, VertSpacing, FontSize;
double     spr, pnt, tickval, bidp, askp, minlot, lswap, sswap, NewSL;
color      FontColor;
datetime   prev_time;
bool       FirstTime, ExtendLine; 

int        nup,ndn;
double     uparr[10],dnarr[10];

//+------------------------------------------------------------------+
int init()  {
//+------------------------------------------------------------------+
  IndiName = "RoomUpDown-" + UniqueID;
  IndicatorShortName(IndiName);

  sym     = Symbol();
  ccy     = Symbol();
  tmf     = Period();
  bidp    = MarketInfo(ccy,MODE_BID);
  askp    = MarketInfo(ccy,MODE_ASK);
  pnt     = MarketInfo(ccy,MODE_POINT);
  dig     = MarketInfo(ccy,MODE_DIGITS);
  spr     = MarketInfo(ccy,MODE_SPREAD);
  tickval = MarketInfo(ccy,MODE_TICKVALUE);
  minlot  = MarketInfo(ccy,MODE_MINLOT);
  lswap   = MarketInfo(ccy,MODE_SWAPLONG);
  sswap   = MarketInfo(ccy,MODE_SWAPSHORT);
  if (dig == 3 || dig == 5) {
    pnt     *= 10;
    spr     /= 10;
    tickval *= 10;
  } 

  if (DownLevels==".")
    DownLevels = UpLevels;
  nup = StrToDoubleArray(UpLevels,uparr);
  ndn = StrToDoubleArray(DownLevels,dnarr);
  
  ExtendLine = false;
  if (LineLength < 0)
    ExtendLine = true;
  LineLength = MathAbs(LineLength); 
  
  RefreshEveryXMins = StrToTF(RefreshPeriod);
  prev_time = -9999;
  vis = GetVisibility(Visibility);
  gv_up = IndiName + Symbol() + "-UP";
  gv_dn = IndiName + Symbol() + "-DN";
  gv_op = IndiName + Symbol() + "-OP";
  return(0);
}

//+------------------------------------------------------------------+
int deinit()  {
//+------------------------------------------------------------------+
  del_obj();
  GlobalVariablesDeleteAll(IndiName+Symbol());
  return(0);
}

//+------------------------------------------------------------------+
int start()  {
//+------------------------------------------------------------------+
  if (RefreshEveryXMins < 0)  {
    if (FirstTime)  {
      del_obj();
      plot_obj();
    }
    FirstTime = false;      
    return(0);
  }  
  if (RefreshEveryXMins == 0) {
    del_obj();
    plot_obj();    
  } else {
    if (prev_time != iTime(sym,RefreshEveryXMins,0))  {
      del_obj();
      plot_obj();
      prev_time = iTime(sym,RefreshEveryXMins,0);
  } }      
  return(0);
}

//+------------------------------------------------------------------+
void del_obj()  {
//+------------------------------------------------------------------+
  int k=0;
  while (k<ObjectsTotal())   {
    string objname = ObjectName(k);
    if (StringSubstr(objname,0,StringLen(IndiName)) == IndiName)  
      ObjectDelete(objname);
    else
      k++;
  }    
  return(0);
}

//+------------------------------------------------------------------+
void plot_obj()   {
//+------------------------------------------------------------------+
  // Calculate ADR.......
  double sum=0, cnt=0;
  for (int k=1; k<=ADRdays; k++)   {
    datetime dt = iTime(ccy,PERIOD_D1,k);
    if (TimeDayOfWeek(dt) == 0)   continue;
    double range = (iHigh(ccy,PERIOD_D1,k) - iLow(ccy,PERIOD_D1,k))/pnt;
    sum += range;
    cnt += 1;
  }
  double adr = DivZero(sum,cnt) * pnt;

  if (MathAbs(StartCandle)>9998)  
    StartCandle = iBarShift(Symbol(),Period(),StrToTime(DateToStr(TimeCurrent(),"Y.M.D' 00:00:00'")));

  for (int i=0; i<nup; i++)   {
    if (FixFromOpen)
      double   level   = iOpen(Symbol(),PERIOD_D1,0) + adr*uparr[i]/100;
    else
      level   = iLow(Symbol(),PERIOD_D1,0) + adr*uparr[i]/100;
    string   objname = IndiName+NumberToStr(i,"'-'Z4'-u'");
    datetime stime   = Time[0]-StartCandle*60*Period();
    datetime etime   = Time[0]-(StartCandle-LineLength)*60*Period();
    string   objtext = NumberToStr(uparr[i],"T+4'%'") + NumberToStr(adr*uparr[i]/100/pnt,"' = 'TR4' pips'") + NumberToStr(level,"' = 'T5.5");
    PlotTL (objname, false, 0, stime, level, etime, level, UpLineColor, LineWidth, LineStyle, ExtendLine, false, vis, objtext);      // Plot trendline
  }  
  
  for (i=0; i<ndn; i++)   {
    if (FixFromOpen)
      level   = iOpen(Symbol(),PERIOD_D1,0) - adr*dnarr[i]/100;
    else
      level   = iHigh(Symbol(),PERIOD_D1,0) - adr*dnarr[i]/100;
    objname = IndiName+NumberToStr(i,"'-'Z4'-d'");
    stime   = Time[0]-StartCandle*60*Period();
    etime   = Time[0]-(StartCandle-LineLength)*60*Period();
    objtext = NumberToStr(-dnarr[i],"T+4'%'") + NumberToStr(adr*dnarr[i]/100/pnt,"' = 'TR4' pips'") + NumberToStr(level,"' = 'T5.5");
    PlotTL (objname, false, 0, stime, level, etime, level, DownLineColor, LineWidth, LineStyle, ExtendLine, false, vis, objtext);      // Plot trendline
  }
  
  if (UseGlobalVariables)   {
    double value = 100*DivZero(Bid-iLow(Symbol(),PERIOD_D1,0),adr);
    GlobalVariableSet(gv_up,value);
    value = 100*DivZero(iHigh(Symbol(),PERIOD_D1,0)-Bid,adr);
    GlobalVariableSet(gv_dn,value);
    value = 100*DivZero(Bid-iOpen(Symbol(),PERIOD_D1,0),adr);
    GlobalVariableSet(gv_op,value);
  }  
  return(0);
}

//+------------------------------------------------------------------+
#include <hanover --- extensible functions (np).mqh>

